#!/bin/sh

. lib/color.sh

FSTYPE=ext3

PARTED=/sbin/parted
PARTED_READ=/sbin/parted
MKFS=/sbin/mkfs.${FSTYPE}
CONFIGDEV=/dev/hda


function fatal {
	echo "${RED}### FATAL: $1${NORMAL}"
	exit 1
}

function savConfig {
	echo -n "  ${BLUE}Saving configuration to $1 ... ${NORMAL}"
	mount $1 /mnt/hd && \
		cp -f /etc/EQOS_CONFIG /mnt/hd/EQOS_CONFIG && \
		umount /mnt/hd
	if [ $? -eq 0 ]; then
		echo "${GREEN}OK${NORMAL}"
	else
		echo "${RED}Failed !${NORMAL}"
	fi
}
function mkCacheDir {
	echo -n "  ${BLUE}Creating block cache directory $1 ... ${NORMAL}"
	mount $1 /mnt/hd && \
		mkdir -p /mnt/hd/knxblock && \
		umount /mnt/hd
	if [ $? -eq 0 ]; then
		echo "${GREEN}OK${NORMAL}"
	else
		echo "${RED}Failed !${NORMAL}"
	fi
}

#########################################################################################"""

echo "${BLUE}Starting disk config utility ..."


# Security test :
if [ ! "x$(hostname)" = 'xeqos' ] ; then
	echo "${RED}Disabling dangerous commands ...${NORMAL}"
	PARTED="echo ${PARTED}"
	MKFS="echo ${MKFS}"
else
	test ! -f /etc/EQOS_CONFIG && fatal "EQOS config not found !"
	. /etc/EQOS_CONFIG
fi

test -n "${KNX_CONFIG_DEVICE}" && echo "  ${GREEN}Config partition detected at boot time on ${YELLOW}${KNX_CONFIG_DEVICE}${GREEN}.${NORMAL}"

test -n "${KNX_CACHE_DEVICE}" && echo "  ${GREEN}Cache partition detected at boot time on ${YELLOW}${KNX_CACHE_DEVICE}${GREEN}.${NORMAL}"

DISKS=$(ls -d1 /sys/block/[sh]d[a-h] 2>/dev/null | sed 's=.*\/\([a-z]\+\)*=\1=')
echo "  ${GREEN}Disks found : ${YELLOW}${DISKS}${GREEN}.${NORMAL}"


KNX_CACHE_PATH='/mnt/tmp'
mkdir -p ${KNX_CACHE_PATH}
		
for DISK in $DISKS; do

	echo "${BLUE}Checking partitions on ${MAGENTA}${DISK}${BLUE} ...${NORMAL}"
	PARTITIONS=$(/bin/ls -d1 /sys/block/${DISK}/${DISK}[0-9]* 2>/dev/null| sed 's=.*\/\([a-z0-9]\+\)*=/dev/\1=')
	
#	if [ $NB_PARTITIONS -gt 0 ] ; then
#		echo "  ${GREEN}System already partitionned ($NB_PARTITIONS partitions).${NORMAL}"

	FOUND_KNX_CACHE_DEVICE=
	
	if test -n "${PARTITIONS}" ; then 
		echo "${CRE}${BLUE}Searching config and cache directory on ${DISK} ...${NORMAL}"
		
		for part in ${PARTITIONS}; do
		
			echo -n "${CRE}${GREEN}Scanning drive in: ${MAGENTA}${part}${NORMAL}"
			
			VOLNAME=$(tune2fs -l ${part} 2>/dev/null| perl -ne 'print $1 if /^Filesystem volume name: +(.*)$/')
			if [ $? -eq 0 ]; then
				if [ "${VOLNAME}" = "EqosConfig" ] ; then
					if [ -z "${KNX_CONFIG_DEVICE}" ]; then
						KNX_CONFIG_DEVICE=${part}
						echo "${CRE}${GREEN}Found new config FS on: ${MAGENTA}${part}${NORMAL}"
						
					elif [ "${KNX_CONFIG_DEVICE}" = "${part}" ]; then
						echo "${CRE}${GREEN}Found config FS on: ${MAGENTA}${part}${NORMAL}"
						
					else
						echo "${CRE}${RED}Found another config FS on: ${MAGENTA}${part}${NORMAL} (first was on ${MAGENTA}${KNX_CONFIG_DEVICE}${NORMAL})"
						
					fi
					
				elif [ "${VOLNAME}" = "EqosCache" ] ; then
					if [ -z "${KNX_CACHE_DEVICE}" ]; then
						KNX_CACHE_DEVICE=${part}
						echo "${CRE}${GREEN}Found new cache FS on: ${MAGENTA}${part}${NORMAL}"
						
					elif [ "${KNX_CACHE_DEVICE}" = "${part}" ]; then
						echo "${CRE}${GREEN}Found cache FS on: ${MAGENTA}${part}${NORMAL}"
						
					else
						echo "${CRE}${RED}Found another cache FS on: ${MAGENTA}${part}${NORMAL} (first was on ${MAGENTA}${KNX_CACHE_DEVICE}${NORMAL})"
						
					fi
					
				else
					echo "${CRE}${GREEN}Found unknown FS on ${MAGENTA}${part}${GREEN}: ${MAGENTA}${VOLNAME}${NORMAL}"
					
				fi
			else
				echo "    ${RED}Unkown filesystem !${NORMAL}"
			fi
		done
	
	else
		rep='n'
		echo "  ${GREEN}No partition detected on ${YELLOW}${DISK}${GREEN}.${NORMAL}"
		echo "${CYAN}Are you sure you want to initialize partition table [yN] ?${NORMAL}"
		read -n1 -s rep
		
		if [ "x$rep" = 'xy' ]; then

			echo -n "${GREEN}Creating a new partition table on ${YELLOW}${DISK}${GREEN}${NORMAL} ..."
			$PARTED -s /dev/${DISK} mklabel msdos
			if [ $? -ne 0 ]; then
				echo "${RED}Failed !${NORMAL}"
				break
			fi
			echo "${GREEN}OK${NORMAL}"
					
			echo -n "${GREEN}Creating config partition on ${YELLOW}${DISK}${GREEN}${NORMAL} ..."
			$PARTED -s /dev/${DISK} mkpart primary 0 50
			if [ $? -ne 0 ]; then
				echo "${RED}Failed !${NORMAL}"
				break
			fi
			echo "${GREEN}OK${NORMAL}"
			
			echo -n "${GREEN}Creating a new file system on ${YELLOW}${DISK}1${GREEN}${NORMAL} ..."
			${MKFS} -m0 -L EqosConfig /dev/${DISK}1 2>/dev/null
			if [ $? -eq 0 ]; then
				echo "${GREEN}OK${NORMAL}"
				KNX_CONFIG_DEVICE=/dev/${DISK}1
				savConfig ${KNX_CONFIG_DEVICE}
			else
				echo "${RED}Failed !${NORMAL}"
			fi
			
			echo -n "${GREEN}Creating a new cache partition on ${YELLOW}${DISK}${GREEN}${NORMAL} ..."
			${PARTED} -s /dev/${DISK} mkpart primary 50 500
			if [ $? -ne 0 ]; then
				echo "${RED}Failed !${NORMAL}"
				break
			fi
			echo "${GREEN}OK${NORMAL}"
			
			echo -n "${GREEN}Creating file system on ${YELLOW}${DISK}2${GREEN}${NORMAL} ..."
			${MKFS} -m0 -L EqosCache /dev/${DISK}2 2>/dev/null
			if [ $? -eq 0 ]; then
				echo "${GREEN}OK${NORMAL}"
				KNX_CACHE_DEVICE=/dev/${DISK}2
				mkCacheDir ${KNX_CACHE_DEVICE}
			else
				echo "${RED}Failed !${NORMAL}"
				break
			fi
			
			
			reboot
		fi
	fi
done

if [ -n "${KNX_CONFIG_DEVICE}" ]; then
	echo "${BLUE}Checking config partition on ${MAGENTA}${KNX_CONFIG_DEVICE}${GREEN}.${NORMAL}"
	STATE=$(tune2fs -l ${KNX_CONFIG_DEVICE} 2>/dev/null| perl -ne 'print $1 if /^Filesystem state: +(.*)$/')
	if [ $? -eq 0 ]; then
		echo "    ${GREEN}Filesystem state : ${MAGENTA}${STATE}${GREEN}.${NORMAL}"

	else
		echo -n "  ${BLUE}Checking filesystem ... ${NORMAL}"
		${FSCK} -f -y ${KNX_CONFIG_DEVICE} >/dev/null 2>&1
		case $? in
			0|1)
				echo "${GREEN}OK${NORMAL}" ;;
			*)
				echo "${RED}Error detected !${NORMAL}"
				echo -n "    ${BLUE}Rebuilding filesystem ... ${NORMAL}"
				${MKFS} -m0 -L EqosConfig ${KNX_CONFIG_DEVICE} 2>/dev/null 2>&1
				savConfig ${KNX_CONFIG_DEVICE}
				;;
		esac
	fi
fi

if [ -n "${KNX_CACHE_DEVICE}" ]; then
	echo "${BLUE}Checking config partition on ${MAGENTA}${KNX_CACHE_DEVICE}${GREEN}.${NORMAL}"
	STATE=$(tune2fs -l ${KNX_CACHE_DEVICE} 2>/dev/null| perl -ne 'print $1 if /^Filesystem state: +(.*)$/')
	if [ $? -eq 0 ]; then
		echo "    ${GREEN}Filesystem state : ${MAGENTA}${STATE}${GREEN}.${NORMAL}"

	else
		echo -n "  ${BLUE}Checking filesystem ... ${NORMAL}"
		if test $(grep -c ${KNX_CACHE_DEVICE} /proc/mounts) -eq 0 ; then
			${FSCK} -f -y ${KNX_CACHE_DEVICE} >/dev/null 2>&1
			case $? in
				0|1)
					echo "${GREEN}OK${NORMAL}" ;;
				*)
					echo "${RED}Error detected !${NORMAL}"
					echo -n "    ${BLUE}Rebuilding filesystem ... ${NORMAL}"
					${MKFS} -m0 -L EqosCache ${KNX_CACHE_DEVICE} 2>/dev/null 2>&1
					;;
			esac
			mkCacheDir ${KNX_CACHE_DEVICE}
		else
			echo "${YELLOW}SKIPPED (Mounted)${NORMAL}"
		fi
	fi
fi

echo "------------------------------------------------------------"
$PARTED -s $CONFIGDEV print
echo "------------------------------------------------------------"





