#!/usr/bin/perl -w

# vim: nu
# vim: set encoding=utf-8
#
# (c) 2006      Jean-Christophe TOUSSAINT <jean-christophe.toussaint@ac-nancy-metz.fr>
#               Stéphane URBANOVSKI <stephane.urbanovski@ac-nancy-metz.fr>
#                                       DSI - Academie de Nancy-Metz
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# you should have received a copy of the GNU General Public License
# along with this program (or with Netsaint);  if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA
#
# $Id: eqos-db.pl,v 1.8 2006-04-20 14:15:49 uid5012 Exp $
#


use strict;
use warnings;
use DBI;
use Getopt::Long;
use Term::ANSIColor qw(:constants);

my $show_help = 0;
my $dbfile = "/var/lib/sqlite/eqosd.sqlite";

my $tableconfig = 0;
my $tablestates = 0;
my $tableservices = 0;
my $tablealerts = 0;

my $insertstates = 0;

my $flushstates = 0;
my $flushconfig = 0;
my $flushservices = 0;
my $flushalerts = 0;
my $flush = 0;

my $reset = 0;

my $sqlite;

my $insert_file;

GetOptions(
		"help" => \$show_help,
		"db=s" => \$dbfile,
		"table-config" => \$tableconfig,
		"table-states" => \$tablestates,
		"table-services" => \$tableservices,
		"table-alerts" => \$tablealerts,
		"insert-states" => \$insertstates,
		"flush-states" => \$flushstates,
		"flush-config" => \$flushconfig,
		"flush-services" => \$flushservices,
		"flush-alerts" => \$flushservices,
		"reset" => \$reset,
		'if=s' => \$insert_file,
	);



if ($show_help == 1) {
	print << "__EOFHELP__";
* outil d'administration de la base de données de eqosd *

	--help           : affiche cet aide
	
	--table-config   : affiche le contenu de la table "eqos_config"
	--table-states   : affiche le contenu de la table "eqos_states"
	--table-services : affiche le contenu de la table "eqos_services"
	--table-alerts : affiche le contenu de la table "eqos_alerts"
	
	--insert-states  : insert des données (echantillons) dans la table "eqos_states"
	
	--flush-states   : purge la table "eqos_states"
	--flush-config   : purge la table "eqos_config"
	--flush-services : purge la table "eqos_services"
	--flush-alerts : purge la table "eqos_alerts"

	--reset          : réinitialise le fichier de base de données !
	
	-if FILE         : execute le fichier sql FILE
	
	par défaut, toutes les tables sont affichées
	
__EOFHELP__
	exit 0;
}

if ($tableconfig == 0 && $tablestates == 0 && $tableservices == 0 && $tablealerts == 0) {
	$tableconfig = 1;
	$tablestates = 1;
	$tableservices = 1;
	$tablealerts = 1;
}

if ($flushstates == 1 || $flushconfig == 1 || $flushservices == 1 || $flushalerts == 1) {
	$flush = 1;
}

if ($reset == 1) {
	print ("ATTENTION ! La réinitialisation de la base de données est très dangereuse pour la stabilité de eqosd si celui-ci est démarré ! Assurez-vous que l'agent EQOS est arrêté !\n");
	print ("Êtes-vous (terriblement) sûr de vouloir dégommer la base de données SQLite $dbfile ? (o/n) ");
	my $ok = <STDIN>;
	chomp $ok;
	if ($ok eq "o") {
		if (system("cat /dev/null > $dbfile") == 0) {
			print "base de données $dbfile atomisée\n";
			exit 0;
		}
		else {
			print "une erreur est survenue pendant la réinitialisation de la base de données\n";
			exit 1;
		}
	}
	else {
		print "action annulée !\n";
	}
	exit 0;
}

$sqlite = DBI->connect("dbi:SQLite:dbname=$dbfile", "", "", { PrintWarn => 1, PrintError => 1, AutoCommit => 1 });



if ( defined($insert_file) && -e $insert_file ) {
	my $error = 0;
	my $i=1;
	open (SQLFILE,"<$insert_file");
	while (my $line = <SQLFILE>) {
		chomp $line;
		print $i++." $line\n";
		$sqlite->do($line) or $error++;
	}
	close (SQLFILE);
	
	if ($error == 0) {
		print "Echantillons insérés\n";
	}else {
		print "Une erreur est survenue pendant l'execution :\n";
		print $sqlite->errstr."\n";
		exit 1;
	}
	exit 0;
}

if ($flush == 1) {
	print ("Êtes-vous sûr de vouloir purger la (ou les) table(s) ? (OUI/n) ");
	my $ok = <STDIN>;
	chomp $ok;
	if ($ok eq "OUI") {
		print "purge lancée\n";
	}
	else {
		print "purge annulée\n";
		exit 0;
	}
}

if ($flushstates == 1) {
	my $error = 0;
	$sqlite->do("DELETE FROM eqos_states") or $error++;
	if ($error == 0) {
		print "table eqos_states purgée\n";
	}
	else {
		print "une erreur est survenue pendant la purge de eqos_states\n";
		exit 1;
	}
}

if ($flushconfig == 1) {
	my $error = 0;
	$sqlite->do("DELETE FROM eqos_config") or $error++;
	if ($error == 0) {
		print "table eqos_config purgée\n";
	}
	else {
		print "une erreur est survenue pendant la purge de eqos_config\n";
		exit 1;
	}
}

if ($flushservices == 1) {
	my $error = 0;
	$sqlite->do("DELETE FROM eqos_services") or $error++;
	if ($error == 0) {
		print "table eqos_services purgée\n";
	}
	else {
		print "une erreur est survenue pendant la purge de eqos_services\n";
		exit 1;
	}
}


if ($flushservices == 1) {
	my $error = 0;
	$sqlite->do("DELETE FROM eqos_alerts") or $error++;
	if ($error == 0) {
		print "table eqos_alerts purgée\n";
	}
	else {
		print "une erreur est survenue pendant la purge de eqos_alerts\n";
		exit 1;
	}
}


if ($flush == 1) {
	exit 0;
}

if ($insertstates == 1) {
	my $error = 0;
	$sqlite->do("INSERT INTO eqos_states VALUES (NULL, " . (time() - 20) . ", 1, 0, 'message1')") or $error++;
	$sqlite->do("INSERT INTO eqos_states VALUES (NULL, " . (time() - 18) . ", 1, 0, 'message2')") or $error++;
	$sqlite->do("INSERT INTO eqos_states VALUES (NULL, " . (time() - 15) . ", 2, 1, 'message3')") or $error++;
	$sqlite->do("INSERT INTO eqos_states VALUES (NULL, " . (time() - 8) . ", 2, 2, 'message4')") or $error++;
	$sqlite->do("INSERT INTO eqos_states VALUES (NULL, " . time() . ", 1, 0, 'message5')" or $error++);
	if ($error == 0) {
		print "echantillons insérés\n";
	}
	else {
		print "une erreur est survenue pendant l'insertion des echantillons\n";
		exit 1;
	}
	exit 0;
}

if ($tableconfig == 1) {
	print "\n" . BOLD WHITE . "* table eqos_config\n" . RESET;
	printf "\%-10s | \%-15s | \%s \n", "id", "name", "value";
	print "-----------+-----------------+----------------------------------------------------------------\n";
	my $resultConfig = $sqlite->selectall_arrayref("SELECT id,name,value FROM eqos_config");
	foreach my $row (@$resultConfig) {
		my ($id,$name,$value) = @$row;
		printf YELLOW . "\%10s" . RESET . " | " . RED . "\%-15s" . RESET . " | " . RED . "\%s" . RESET . "\n", $id, $name, $value;
	}
}

if ($tablestates == 1) {
	print "\n" . BOLD WHITE . "* table eqos_states\n" . RESET;
	printf "\%-10s | \%-10s | \%-19s | \%-10s | \%-10s | \%s \n", "id", "timestamp", "date (from ts)", "idService", "state", "message";
	print "-----------+------------+---------------------+------------+------------+---------------------\n";
	my $resultStates = $sqlite->selectall_arrayref("SELECT id,timestamp,idService,state,message FROM eqos_states");
	foreach my $row (@$resultStates) {
		my ($id,$timestamp,$idService,$state,$message) = @$row;
		my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday) = gmtime($timestamp);
		$year += 1900;
		printf YELLOW . "\%10s" . RESET . " | " . YELLOW . "\%10s" . RESET . " | " . RED . "\%10s" . RESET . " | " . YELLOW . "\%10s" . RESET . " | " . YELLOW . "\%10s" . RESET . " | " . RED . "\%s" . RESET . "\n", $id, $timestamp, sprintf("%02d/%02d/%04d %02d:%02d:%02d", $mday, $mon, $year, $hour, $min, $sec), $idService, $state, $message;
	}
}

if ($tableservices == 1) {
	print "\n" . BOLD WHITE . "* table eqos_services\n" . RESET;
	printf "\%-10s | \%-25s | \%-8s | \%s \n", "id", "plugin", "period", "args";
	print "-----------+---------------------------+----------+-------------------------------------------\n";
	my $resultConfig = $sqlite->selectall_arrayref("SELECT id,plugin,period,args FROM eqos_services");
	foreach my $row (@$resultConfig) {
		my ($id, $plugin, $period, $args) = @$row;
		printf YELLOW . "\%10s" . RESET . " | " . RED . "\%-25s" . RESET . " | " . YELLOW . "\%8s" . RESET . " | " . RED . "\%s" . RESET . "\n", $id, $plugin, $period, $args;
	}
}

if ($tablealerts == 1) {
	print "\n" . BOLD WHITE . "* table eqos_alerts\n" . RESET;
	printf "\%-10s | \%-15s | \%-8s | \%s \n", "id", "timestamp", "module", "message";
	print "-----------+-----------------+----------+-----------------------------------------------------\n";
	my $resultConfig = $sqlite->selectall_arrayref("SELECT id,timestamp,module,message FROM eqos_alerts");
	foreach my $row (@$resultConfig) {
		my ($id, $timestamp, $module, $message) = @$row;
		printf YELLOW . "\%10s" . RESET . " | " . YELLOW . "\%-15s" . RESET . " | " . RED . "\%-8s" . RESET . " | " . RED . "\%s" . RESET . "\n", $id, $timestamp, $module, $message;
	}
}


$sqlite->disconnect();

exit 0;

