#!/usr/bin/perl -w
# vim: nu
# vim: set encoding=utf-8
#
# (c) 2006      Jean-Christophe TOUSSAINT <jean-christophe.toussaint@ac-nancy-metz.fr>
#               Stéphane URBANOVSKI <stephane.urbanovski@ac-nancy-metz.fr>
#                                       DSI - Académie de Nancy-Metz
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# you should have received a copy of the GNU General Public License
# along with this program (or with Netsaint);  if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA
#
# $Id: libEqosAgent.pm,v 1.1 2006-03-09 14:55:19 uid5000 Exp $

package libEqosAgent;

use strict;					# should never be differently :-)
use warnings;
use Carp;

# Init Exporter :
require Exporter;
use vars qw(@ISA @EXPORT @EXPORT_OK );
@ISA = qw(Exporter);


# Set exported vars and subs :
@EXPORT = qw( &make_log &read_text &write_text $debuglevel $globalname);
@EXPORT_OK = qw(%EXIT_CODES %ERR_LEVELS %EXIT_CODES_REV);

# Other common libs :
use Term::ANSIColor qw(:constants);

# use vars qw(%EXIT_CODES %ERR_LEVELS $debuglevel);
our $debuglevel = 2;

# global name
our $globalname = "";

# Predefined exit codes for Nagios:
our %EXIT_CODES = (
	'UNKNOWN'	=> -1,
	'OK' 		=> 0,
	'WARNING'	=> 1,
	'CRITICAL'	=> 2
	);
	
our %EXIT_CODES_REV = (
	-1	=> 'UNKNOWN',
	0	=> 'OK',
	1	=> 'WARNING',
	2	=> 'CRITICAL',
	3	=> 'UNKNOWN',
	);

	
# Predefined log levels:
our %ERR_LEVELS = (
	'EMERG'		=> 0,	# systeme inutilisable
	'ALERT'		=> 1,	# action a effectuer immediatement
	'CRIT'		=> 2,	# conditions critiques
	'ERR'		=> 3,	# conditions d'erreurs
	'WARNING'	=> 4,	# message d'avertissement
	'NOTICE'	=> 5,	# normal mais significatif
	'INFO'		=> 6,	# informations
	'DEBUG'		=> 7,	# messages de debugging
	'TEST'		=> 8	# for testing only
);

# Write a dated and formated log message
sub make_log {
	my ($lvl, $str) = @_;
	if ($lvl <= $debuglevel) {
		my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday) = gmtime();
		print WHITE . sprintf("%02d/%02d %02d:%02d:%02d", $mday, $mon, $hour, $min, $sec) . " | [$globalname] $str" . RESET . "\n";
	}
}

# lit un fichier texte
sub read_text($) {
	my($filename) = @_;
	my $output = "";

	my $fh = new IO::File;
	if ($fh->open("< $filename")) {
		while (<$fh>) {
			$output .= $_;
		}
		$fh->close;
	}
	else {
		&make_log (1, BOLD RED . "can't read $filename");
		$output = undef;
	}
	return $output;
}

# écrit un fichier texte
sub write_text($;$) {
	my($filename, $data) = @_;
	my $fh = new IO::File;
	if ($fh->open("> $filename")) {
		print $fh $data;
		$fh->close;
		return 1;
	}
	else {
		&make_log (1, BOLD RED . "can't write $filename");
		return 0;
	}
}

1;
